@@ -25,9 +25,7 @@
 #include <QDir>
 #include <QFile>
 #include <QPointF>
-#include <QSharedPointer>
 #include <QTemporaryFile>
-#include <QtGlobal>
 
 /*
         From Wikipedia: The Open Packaging Conventions (OPC) is a
@@ -74,8 +72,8 @@ std::string copyTag(const std::string &sFrom, const std::string &sTo, const std:
     const std::string tagEnd         = "</" + tag + ">";
 
     // search all occurrences of tag in 'sFrom'
-    std::string sFromData = "";
-    size_t startIndex     = 0;
+    std::string sFromData;
+    size_t startIndex = 0;
     while (true) {
         std::size_t startPos = sFrom.find(tagToFindStart, startIndex);
         if (startPos != std::string::npos) {
@@ -164,8 +162,8 @@ void DocumentPrivate::init()
     if (!contentTypes)
         contentTypes = std::make_shared<ContentTypes>(ContentTypes::F_NewFromScratch);
 
-    if (workbook.isNull())
-        workbook = QSharedPointer<Workbook>(new Workbook(Workbook::F_NewFromScratch));
+    if (!workbook)
+        workbook = std::shared_ptr<Workbook>(new Workbook(Workbook::F_NewFromScratch));
 }
 
 bool DocumentPrivate::loadPackage(QIODevice *device)
@@ -218,7 +216,7 @@ bool DocumentPrivate::loadPackage(QIODevice *device)
 
     // load workbook now, Get the workbook file path from the root rels file
     // In normal case, this should be "xl/workbook.xml"
-    workbook = QSharedPointer<Workbook>(new Workbook(Workbook::F_LoadFromExists));
+    workbook = std::shared_ptr<Workbook>(new Workbook(Workbook::F_LoadFromExists));
     QList<XlsxRelationship> rels_xl =
         rootRels.documentRelationships(QStringLiteral("/officeDocument"));
     if (rels_xl.isEmpty())
@@ -248,7 +246,7 @@ bool DocumentPrivate::loadPackage(QIODevice *device)
             path = xlworkbook_Dir + QLatin1String("/") + name;
         }
 
-        QSharedPointer<Styles> styles(new Styles(Styles::F_LoadFromExists));
+        std::shared_ptr<Styles> styles(new Styles(Styles::F_LoadFromExists));
         styles->loadFromXmlData(zipReader.fileData(path));
         workbook->d_func()->styles = styles;
     }
@@ -286,7 +284,7 @@ bool DocumentPrivate::loadPackage(QIODevice *device)
 
     // load external links
     for (int i = 0; i < workbook->d_func()->externalLinks.count(); ++i) {
-        SimpleOOXmlFile *link = workbook->d_func()->externalLinks[i].data();
+        SimpleOOXmlFile *link = workbook->d_func()->externalLinks[i].get();
         QString rel_path      = getRelFilePath(link->filePath());
         // If the .rel file exists, load it.
         if (zipReader.filePaths().contains(rel_path))
@@ -304,9 +302,9 @@ bool DocumentPrivate::loadPackage(QIODevice *device)
     }
 
     // load charts
-    QList<QSharedPointer<Chart>> chartFileToLoad = workbook->chartFiles();
+    QList<std::shared_ptr<Chart>> chartFileToLoad = workbook->chartFiles();
     for (int i = 0; i < chartFileToLoad.size(); ++i) {
-        QSharedPointer<Chart> cf = chartFileToLoad[i];
+        std::shared_ptr<Chart> cf = chartFileToLoad[i];
         cf->loadFromXmlData(zipReader.fileData(cf->filePath()));
     }
 
@@ -336,13 +334,13 @@ bool DocumentPrivate::savePackage(QIODevice *device) const
     DocPropsCore docPropsCore(DocPropsCore::F_NewFromScratch);
 
     // save worksheet xml files
-    QList<QSharedPointer<AbstractSheet>> worksheets =
+    QList<std::shared_ptr<AbstractSheet>> worksheets =
         workbook->getSheetsByTypes(AbstractSheet::ST_WorkSheet);
     if (!worksheets.isEmpty())
         docPropsApp.addHeadingPair(QStringLiteral("Worksheets"), worksheets.size());
 
     for (int i = 0; i < worksheets.size(); ++i) {
-        QSharedPointer<AbstractSheet> sheet = worksheets[i];
+        std::shared_ptr<AbstractSheet> sheet = worksheets[i];
         contentTypes->addWorksheetName(QStringLiteral("sheet%1").arg(i + 1));
         docPropsApp.addPartTitle(sheet->sheetName());
 
@@ -356,12 +354,12 @@ bool DocumentPrivate::savePackage(QIODevice *device) const
     }
 
     // save chartsheet xml files
-    QList<QSharedPointer<AbstractSheet>> chartsheets =
+    QList<std::shared_ptr<AbstractSheet>> chartsheets =
         workbook->getSheetsByTypes(AbstractSheet::ST_ChartSheet);
     if (!chartsheets.isEmpty())
         docPropsApp.addHeadingPair(QStringLiteral("Chartsheets"), chartsheets.size());
     for (int i = 0; i < chartsheets.size(); ++i) {
-        QSharedPointer<AbstractSheet> sheet = chartsheets[i];
+        std::shared_ptr<AbstractSheet> sheet = chartsheets[i];
         contentTypes->addWorksheetName(QStringLiteral("sheet%1").arg(i + 1));
         docPropsApp.addPartTitle(sheet->sheetName());
 
@@ -375,7 +373,7 @@ bool DocumentPrivate::savePackage(QIODevice *device) const
 
     // save external links xml files
     for (int i = 0; i < workbook->d_func()->externalLinks.count(); ++i) {
-        SimpleOOXmlFile *link = workbook->d_func()->externalLinks[i].data();
+        SimpleOOXmlFile *link = workbook->d_func()->externalLinks[i].get();
         contentTypes->addExternalLinkName(QStringLiteral("externalLink%1").arg(i + 1));
 
         zipWriter.addFile(QStringLiteral("xl/externalLinks/externalLink%1.xml").arg(i + 1),
@@ -438,7 +436,7 @@ bool DocumentPrivate::savePackage(QIODevice *device) const
     // save chart xml files
     for (int i = 0; i < workbook->chartFiles().size(); ++i) {
         contentTypes->addChartName(QStringLiteral("chart%1").arg(i + 1));
-        QSharedPointer<Chart> cf = workbook->chartFiles()[i];
+        std::shared_ptr<Chart> cf = workbook->chartFiles()[i];
         zipWriter.addFile(QStringLiteral("xl/charts/chart%1.xml").arg(i + 1), cf->saveToXmlData());
     }
 
@@ -470,6 +468,97 @@ bool DocumentPrivate::savePackage(QIODevice *device) const
     return true;
 }
 
+//
+// j2doll/csv branch
+//
+// Save from XLSX to CSV
+bool DocumentPrivate::saveCsv(QString mainCSVFileName) const
+{
+    Q_Q(const Document);
+
+    int sheetIndexNumber  = 0;
+    const auto sheetNames = q->sheetNames();
+    for (const auto &curretnSheetName : sheetNames) {
+
+        QXlsx::AbstractSheet *currentSheet = q->sheet(curretnSheetName);
+        if (currentSheet == nullptr) {
+            continue;
+        }
+
+        // get full cells of sheet
+        int maxRow = -1;
+        int maxCol = -1;
+
+        currentSheet->workbook()->setActiveSheet(sheetIndexNumber);
+
+        auto wsheet = static_cast<Worksheet *>(currentSheet->workbook()->activeSheet());
+        if (wsheet == nullptr) {
+            continue;
+        }
+
+        QString strSheetName = wsheet->sheetName(); // sheet name
+
+        QVector<QVector<QString>> cellValues;
+        for (int rc = 0; rc < maxRow; rc++) {
+            QVector<QString> tempValue;
+
+            for (int cc = 0; cc < maxCol; cc++) {
+                tempValue.push_back(QString{});
+            }
+
+            cellValues.push_back(tempValue);
+        }
+
+        const QVector<CellLocation> clList = wsheet->getFullCells(&maxRow, &maxCol);
+        for (const auto &cl : clList) {
+            int row = cl.row - 1;
+            int col = cl.col - 1;
+
+            std::shared_ptr<Cell> ptrCell = cl.cell; // cell pointer
+            QVariant var                  = ptrCell->value();
+            QString str                   = var.toString();
+
+            cellValues[row][col] = str;
+        }
+
+        // TODO:
+        //  (1) save as csv file name (using { mainCSVFileName + strSheetName })
+
+        QString csvFileName = mainCSVFileName + u'_' + strSheetName + QLatin1String(".csv");
+        QFile csvFile(csvFileName);
+        if (!csvFile.open(QIODevice::WriteOnly)) {
+            continue;
+        }
+
+        //  (2) save sheet values
+        //     such as  A,,B,,,,C,,,D,,
+
+        for (int rc = 0; rc < maxRow; rc++) {
+            for (int cc = 0; cc < maxCol; cc++) {
+
+                QString cellData = cellValues[rc][cc];
+
+                if (cellData.size() >= 0) {
+                    csvFile.write(cellData.toUtf8()); // cell data
+                }
+
+                csvFile.write(","); // delimeter
+            }
+
+            csvFile.write("\n"); // CR
+
+            csvFile.flush();
+        }
+
+        // file.flush();
+
+        csvFile.close();
+
+    } // foreach (QString curretnSheetName, q->sheetNames()) ...
+
+    return true;
+}
+
 bool DocumentPrivate::copyStyle(const QString &from, const QString &to)
 {
     // create a temp file because the zip writer cannot modify already existing zips
@@ -483,7 +572,7 @@ bool DocumentPrivate::copyStyle(const QString &from, const QString &to)
     ZipReader zipReader(from);
     QStringList filePaths = zipReader.filePaths();
 
-    QSharedPointer<ZipReader> toReader = QSharedPointer<ZipReader>(new ZipReader(to));
+    auto toReader = std::make_shared<ZipReader>(to);
 
     QStringList toFilePaths = toReader->filePaths();
 
@@ -538,7 +627,7 @@ bool DocumentPrivate::copyStyle(const QString &from, const QString &to)
 
     temporalZip.close();
 
-    toReader.clear();
+    toReader.reset();
 
     tempFile.close();
 
@@ -1008,7 +1097,7 @@ bool Document::addConditionalFormatting(const ConditionalFormatting &cf)
  *
  * \sa read()
  */
-Cell *Document::cellAt(const CellReference &pos) const
+std::shared_ptr<Cell> Document::cellAt(const CellReference &pos) const
 {
     if (Worksheet *sheet = currentWorksheet())
         return sheet->cellAt(pos);
@@ -1021,11 +1110,11 @@ Cell *Document::cellAt(const CellReference &pos) const
  *
  * \sa read()
  */
-Cell *Document::cellAt(int row, int col) const
+std::shared_ptr<Cell> Document::cellAt(int row, int col) const
 {
     if (Worksheet *sheet = currentWorksheet())
         return sheet->cellAt(row, col);
-    return nullptr;
+    return {};
 }
 
 /*!
@@ -1113,7 +1202,7 @@ QStringList Document::documentPropertyNames() const
 Workbook *Document::workbook() const
 {
     Q_D(const Document);
-    return d->workbook.data();
+    return d->workbook.get();
 }
 
 /*!
@@ -1278,6 +1367,13 @@ bool Document::saveAs(QIODevice *device) const
     return d->savePackage(device);
 }
 
+bool Document::saveAsCsv(const QString mainCSVFileName) const
+{
+    Q_D(const Document);
+
+    return d->saveCsv(mainCSVFileName);
+}
+
 bool Document::isLoadPackage() const
 {
     Q_D(const Document);
@@ -1443,7 +1539,7 @@ bool Document::autosizeColumnWidth(int colFirst, int colLast)
   Auto sets width in characters for all columns.
   Returns true on success.
  */
-bool Document::autosizeColumnWidth(void)
+bool Document::autosizeColumnWidth()
 {
     bool erg = false;
 
